XK_XF86_ModeLock = 0x1008FF01

XK_XF86_MonBrightnessUp    = 0x1008FF02
XK_XF86_MonBrightnessDown  = 0x1008FF03
XK_XF86_KbdLightOnOff      = 0x1008FF04
XK_XF86_KbdBrightnessUp    = 0x1008FF05
XK_XF86_KbdBrightnessDown  = 0x1008FF06
XK_XF86_MonBrightnessCycle = 0x1008FF07

XK_XF86_Standby          = 0x1008FF10
XK_XF86_AudioLowerVolume = 0x1008FF11
XK_XF86_AudioMute        = 0x1008FF12
XK_XF86_AudioRaiseVolume = 0x1008FF13
XK_XF86_AudioPlay        = 0x1008FF14
XK_XF86_AudioStop        = 0x1008FF15
XK_XF86_AudioPrev        = 0x1008FF16
XK_XF86_AudioNext        = 0x1008FF17
XK_XF86_HomePage         = 0x1008FF18
XK_XF86_Mail             = 0x1008FF19
XK_XF86_Start            = 0x1008FF1A
XK_XF86_Search           = 0x1008FF1B
XK_XF86_AudioRecord      = 0x1008FF1C

XK_XF86_Calculator      = 0x1008FF1D
XK_XF86_Memo            = 0x1008FF1E
XK_XF86_ToDoList        = 0x1008FF1F
XK_XF86_Calendar        = 0x1008FF20
XK_XF86_PowerDown       = 0x1008FF21
XK_XF86_ContrastAdjust  = 0x1008FF22
XK_XF86_RockerUp        = 0x1008FF23
XK_XF86_RockerDown      = 0x1008FF24
XK_XF86_RockerEnter     = 0x1008FF25

XK_XF86_Back            = 0x1008FF26
XK_XF86_Forward         = 0x1008FF27
XK_XF86_Stop            = 0x1008FF28
XK_XF86_Refresh         = 0x1008FF29
XK_XF86_PowerOff        = 0x1008FF2A
XK_XF86_WakeUp          = 0x1008FF2B
XK_XF86_Eject           = 0x1008FF2C
XK_XF86_ScreenSaver     = 0x1008FF2D
XK_XF86_WWW             = 0x1008FF2E
XK_XF86_Sleep           = 0x1008FF2F
XK_XF86_Favorites       = 0x1008FF30
XK_XF86_AudioPause      = 0x1008FF31
XK_XF86_AudioMedia      = 0x1008FF32
XK_XF86_MyComputer      = 0x1008FF33
XK_XF86_VendorHome      = 0x1008FF34
XK_XF86_LightBulb       = 0x1008FF35
XK_XF86_Shop            = 0x1008FF36
XK_XF86_History         = 0x1008FF37
XK_XF86_OpenURL         = 0x1008FF38
XK_XF86_AddFavorite     = 0x1008FF39
XK_XF86_HotLinks        = 0x1008FF3A
XK_XF86_BrightnessAdjust = 0x1008FF3B
XK_XF86_Finance         = 0x1008FF3C
XK_XF86_Community       = 0x1008FF3D
XK_XF86_AudioRewind     = 0x1008FF3E
XK_XF86_XF86BackForward = 0x1008FF3F
XK_XF86_Launch0         = 0x1008FF40
XK_XF86_Launch1         = 0x1008FF41
XK_XF86_Launch2         = 0x1008FF42
XK_XF86_Launch3         = 0x1008FF43
XK_XF86_Launch4         = 0x1008FF44
XK_XF86_Launch5         = 0x1008FF45
XK_XF86_Launch6         = 0x1008FF46
XK_XF86_Launch7         = 0x1008FF47
XK_XF86_Launch8         = 0x1008FF48
XK_XF86_Launch9         = 0x1008FF49
XK_XF86_LaunchA         = 0x1008FF4A
XK_XF86_LaunchB         = 0x1008FF4B
XK_XF86_LaunchC         = 0x1008FF4C
XK_XF86_LaunchD         = 0x1008FF4D
XK_XF86_LaunchE         = 0x1008FF4E
XK_XF86_LaunchF         = 0x1008FF4F

XK_XF86_ApplicationLeft = 0x1008FF50
XK_XF86_ApplicationRight = 0x1008FF51
XK_XF86_Book            = 0x1008FF52
XK_XF86_CD              = 0x1008FF53
XK_XF86_Calculater      = 0x1008FF54
XK_XF86_Clear           = 0x1008FF55
XK_XF86_Close           = 0x1008FF56
XK_XF86_Copy            = 0x1008FF57
XK_XF86_Cut             = 0x1008FF58
XK_XF86_Display         = 0x1008FF59
XK_XF86_DOS             = 0x1008FF5A
XK_XF86_Documents       = 0x1008FF5B
XK_XF86_Excel           = 0x1008FF5C
XK_XF86_Explorer        = 0x1008FF5D
XK_XF86_Game            = 0x1008FF5E
XK_XF86_Go              = 0x1008FF5F
XK_XF86_iTouch          = 0x1008FF60
XK_XF86_LogOff          = 0x1008FF61
XK_XF86_Market          = 0x1008FF62
XK_XF86_Meeting         = 0x1008FF63
XK_XF86_MenuKB          = 0x1008FF65
XK_XF86_MenuPB          = 0x1008FF66
XK_XF86_MySites         = 0x1008FF67
XK_XF86_New             = 0x1008FF68
XK_XF86_News            = 0x1008FF69
XK_XF86_OfficeHome      = 0x1008FF6A
XK_XF86_Open            = 0x1008FF6B
XK_XF86_Option          = 0x1008FF6C
XK_XF86_Paste           = 0x1008FF6D
XK_XF86_Phone           = 0x1008FF6E
XK_XF86_Q               = 0x1008FF70
XK_XF86_Reply           = 0x1008FF72
XK_XF86_Reload          = 0x1008FF73
XK_XF86_RotateWindows   = 0x1008FF74
XK_XF86_RotationPB      = 0x1008FF75
XK_XF86_RotationKB      = 0x1008FF76
XK_XF86_Save            = 0x1008FF77
XK_XF86_ScrollUp        = 0x1008FF78
XK_XF86_ScrollDown      = 0x1008FF79
XK_XF86_ScrollClick     = 0x1008FF7A
XK_XF86_Send            = 0x1008FF7B
XK_XF86_Spell           = 0x1008FF7C
XK_XF86_SplitScreen     = 0x1008FF7D
XK_XF86_Support         = 0x1008FF7E
XK_XF86_TaskPane        = 0x1008FF7F
XK_XF86_Terminal        = 0x1008FF80
XK_XF86_Tools           = 0x1008FF81
XK_XF86_Travel          = 0x1008FF82
XK_XF86_UserPB          = 0x1008FF84
XK_XF86_User1KB         = 0x1008FF85
XK_XF86_User2KB         = 0x1008FF86
XK_XF86_Video           = 0x1008FF87
XK_XF86_WheelButton     = 0x1008FF88
XK_XF86_Word            = 0x1008FF89
XK_XF86_Xfer            = 0x1008FF8A
XK_XF86_ZoomIn          = 0x1008FF8B
XK_XF86_ZoomOut         = 0x1008FF8C

XK_XF86_Away            = 0x1008FF8D
XK_XF86_Messenger       = 0x1008FF8E
XK_XF86_WebCam          = 0x1008FF8F
XK_XF86_MailForward     = 0x1008FF90
XK_XF86_Pictures        = 0x1008FF91
XK_XF86_Music           = 0x1008FF92

XK_XF86_Battery         = 0x1008FF93
XK_XF86_Bluetooth       = 0x1008FF94
XK_XF86_WLAN            = 0x1008FF95
XK_XF86_UWB             = 0x1008FF96

XK_XF86_AudioForward    = 0x1008FF97
XK_XF86_AudioRepeat     = 0x1008FF98
XK_XF86_AudioRandomPlay = 0x1008FF99
XK_XF86_Subtitle        = 0x1008FF9A
XK_XF86_AudioCycleTrack = 0x1008FF9B
XK_XF86_CycleAngle      = 0x1008FF9C
XK_XF86_FrameBack       = 0x1008FF9D
XK_XF86_FrameForward    = 0x1008FF9E
XK_XF86_Time            = 0x1008FF9F
XK_XF86_Select          = 0x1008FFA0
XK_XF86_View            = 0x1008FFA1
XK_XF86_TopMenu         = 0x1008FFA2

XK_XF86_Red             = 0x1008FFA3
XK_XF86_Green           = 0x1008FFA4
XK_XF86_Yellow          = 0x1008FFA5
XK_XF86_Blue            = 0x1008FFA6

XK_XF86_Suspend         = 0x1008FFA7
XK_XF86_Hibernate       = 0x1008FFA8
XK_XF86_TouchpadToggle  = 0x1008FFA9
XK_XF86_TouchpadOn      = 0x1008FFB0
XK_XF86_TouchpadOff     = 0x1008FFB1

XK_XF86_AudioMicMute    = 0x1008FFB2

XK_XF86_Keyboard        = 0x1008FFB3

XK_XF86_WWAN            = 0x1008FFB4
XK_XF86_RFKill          = 0x1008FFB5

XK_XF86_AudioPreset     = 0x1008FFB6

XK_XF86_RotationLockToggle = 0x1008FFB7

XK_XF86_FullScreen      = 0x1008FFB8

XK_XF86_Switch_VT_1     = 0x1008FE01
XK_XF86_Switch_VT_2     = 0x1008FE02
XK_XF86_Switch_VT_3     = 0x1008FE03
XK_XF86_Switch_VT_4     = 0x1008FE04
XK_XF86_Switch_VT_5     = 0x1008FE05
XK_XF86_Switch_VT_6     = 0x1008FE06
XK_XF86_Switch_VT_7     = 0x1008FE07
XK_XF86_Switch_VT_8     = 0x1008FE08
XK_XF86_Switch_VT_9     = 0x1008FE09
XK_XF86_Switch_VT_10    = 0x1008FE0A
XK_XF86_Switch_VT_11    = 0x1008FE0B
XK_XF86_Switch_VT_12    = 0x1008FE0C

XK_XF86_Ungrab          = 0x1008FE20
XK_XF86_ClearGrab       = 0x1008FE21
XK_XF86_Next_VMode      = 0x1008FE22
XK_XF86_Prev_VMode      = 0x1008FE23
XK_XF86_LogWindowTree	= 0x1008FE24
XK_XF86_LogGrabInfo	= 0x1008FE25



# Xlib.ext.dpms -- X Display Power Management Signaling
#
#    Copyright (C) 2020 Thiago Kenji Okada <thiagokokada@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

'''
This extension provides X Protocol control over the VESA Display
Power Management Signaling (DPMS) characteristics of video boards
under control of the X Window System.

Documentation: https://www.x.org/releases/X11R7.7/doc/xextproto/dpms.html
'''

from Xlib.protocol import rq

extname = 'DPMS'


# DPMS Extension Power Levels
#      0     DPMSModeOn          In use
#      1     DPMSModeStandby     Blanked, low power
#      2     DPMSModeSuspend     Blanked, lower power
#      3     DPMSModeOff         Shut off, awaiting activity
DPMSModeOn = 0
DPMSModeStandby = 1
DPMSModeSuspend = 2
DPMSModeOff = 3

DPMSPowerLevel = (
    DPMSModeOn,
    DPMSModeStandby,
    DPMSModeSuspend,
    DPMSModeOff,
)


class DPMSGetVersion(rq.ReplyRequest):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(0),
        rq.RequestLength(),
        rq.Card16('major_version'),
        rq.Card16('minor_version'),
        )

    _reply = rq.Struct(
            rq.ReplyCode(),
            rq.Pad(1),
            rq.Card16('sequence_number'),
            rq.ReplyLength(),
            rq.Card16('major_version'),
            rq.Card16('minor_version'),
            rq.Pad(20),
            )


def get_version(self):
    return DPMSGetVersion(display=self.display,
                          opcode=self.display.get_extension_major(extname),
                          major_version=1,
                          minor_version=1)


class DPMSCapable(rq.ReplyRequest):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(1),
        rq.RequestLength(),
        )

    _reply = rq.Struct(
            rq.ReplyCode(),
            rq.Pad(1),
            rq.Card16('sequence_number'),
            rq.ReplyLength(),
            rq.Bool('capable'),
            rq.Pad(23),
            )


def capable(self):
    return DPMSCapable(display=self.display,
                       opcode=self.display.get_extension_major(extname),
                       major_version=1,
                       minor_version=1)


class DPMSGetTimeouts(rq.ReplyRequest):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(2),
        rq.RequestLength(),
        )

    _reply = rq.Struct(
            rq.ReplyCode(),
            rq.Pad(1),
            rq.Card16('sequence_number'),
            rq.ReplyLength(),
            rq.Card16('standby_timeout'),
            rq.Card16('suspend_timeout'),
            rq.Card16('off_timeout'),
            rq.Pad(18),
            )


def get_timeouts(self):
    return DPMSGetTimeouts(display=self.display,
                           opcode=self.display.get_extension_major(extname),
                           major_version=1,
                           minor_version=1)


class DPMSSetTimeouts(rq.Request):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(3),
        rq.RequestLength(),
        rq.Card16('standby_timeout'),
        rq.Card16('suspend_timeout'),
        rq.Card16('off_timeout'),
        rq.Pad(2)
        )


def set_timeouts(self, standby_timeout, suspend_timeout, off_timeout):
    return DPMSSetTimeouts(display=self.display,
                           opcode=self.display.get_extension_major(extname),
                           major_version=1,
                           minor_version=1,
                           standby_timeout=standby_timeout,
                           suspend_timeout=suspend_timeout,
                           off_timeout=off_timeout)


class DPMSEnable(rq.Request):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(4),
        rq.RequestLength(),
        )


def enable(self):
    return DPMSEnable(display=self.display,
                      opcode=self.display.get_extension_major(extname),
                      major_version=1,
                      minor_version=1)


class DPMSDisable(rq.Request):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(5),
        rq.RequestLength(),
        )


def disable(self):
    return DPMSDisable(display=self.display,
                       opcode=self.display.get_extension_major(extname),
                       major_version=1,
                       minor_version=1)


class DPMSForceLevel(rq.Request):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(6),
        rq.RequestLength(),
        rq.Resource('power_level', DPMSPowerLevel),
        )


def force_level(self, power_level):
    return DPMSForceLevel(display=self.display,
                          opcode=self.display.get_extension_major(extname),
                          major_version=1,
                          minor_version=1,
                          power_level=power_level)


class DPMSInfo(rq.ReplyRequest):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(7),
        rq.RequestLength(),
        )

    _reply = rq.Struct(
        rq.ReplyCode(),
        rq.Pad(1),
        rq.Card16('sequence_number'),
        rq.ReplyLength(),
        rq.Card16('power_level'),
        rq.Bool('state'),
        rq.Pad(21),
        )


def info(self):
    return DPMSInfo(display=self.display,
                    opcode=self.display.get_extension_major(extname),
                    major_version=1,
                    minor_version=1)


def init(disp, _info):
    disp.extension_add_method('display', 'dpms_get_version', get_version)
    disp.extension_add_method('display', 'dpms_capable', capable)
    disp.extension_add_method('display', 'dpms_get_timeouts', get_timeouts)
    disp.extension_add_method('display', 'dpms_set_timeouts', set_timeouts)
    disp.extension_add_method('display', 'dpms_enable', enable)
    disp.extension_add_method('display', 'dpms_disable', disable)
    disp.extension_add_method('display', 'dpms_force_level', force_level)
    disp.extension_add_method('display', 'dpms_info', info)



# Xlib.ext.xfixes -- XFIXES extension module
#
#    Copyright (C) 2010-2011 Outpost Embedded, LLC
#      Forest Bond <forest.bond@rapidrollout.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

'''
A partial implementation of the XFIXES extension.  Only the HideCursor and
ShowCursor requests and SelectionNotify events are provided.
'''

from Xlib.protocol import rq

extname = 'XFIXES'

XFixesSelectionNotify                   = 0
XFixesCursorNotify                      = 1

XFixesSetSelectionOwnerNotifyMask       = (1 << 0)
XFixesSelectionWindowDestroyNotifyMask  = (1 << 1)
XFixesSelectionClientCloseNotifyMask    = (1 << 2)
XFixesDisplayCursorNotifyMask           = (1 << 0)

XFixesSetSelectionOwnerNotify           = 0
XFixesSelectionWindowDestroyNotify      = 1
XFixesSelectionClientCloseNotify        = 2
XFixesDisplayCursorNotify               = 0

class QueryVersion(rq.ReplyRequest):
    _request = rq.Struct(rq.Card8('opcode'),
                         rq.Opcode(0),
                         rq.RequestLength(),
                         rq.Card32('major_version'),
                         rq.Card32('minor_version')
                         )
    _reply = rq.Struct(rq.ReplyCode(),
                       rq.Pad(1),
                       rq.Card16('sequence_number'),
                       rq.ReplyLength(),
                       rq.Card32('major_version'),
                       rq.Card32('minor_version'),
                       rq.Pad(16)
                       )


def query_version(self):
    return QueryVersion(display=self.display,
                        opcode=self.display.get_extension_major(extname),
                        major_version=4,
                        minor_version=0)


class HideCursor(rq.Request):
    _request = rq.Struct(rq.Card8('opcode'),
                         rq.Opcode(29),
                         rq.RequestLength(),
                         rq.Window('window')
                         )

def hide_cursor(self):
    HideCursor(display=self.display,
               opcode=self.display.get_extension_major(extname),
               window=self)


class ShowCursor(rq.Request):
    _request = rq.Struct(rq.Card8('opcode'),
                         rq.Opcode(30),
                         rq.RequestLength(),
                         rq.Window('window')
                         )


def show_cursor(self):
    ShowCursor(display=self.display,
               opcode=self.display.get_extension_major(extname),
               window=self)

class SelectSelectionInput(rq.Request):
    _request = rq.Struct(rq.Card8('opcode'),
                         rq.Opcode(2),
                         rq.RequestLength(),
                         rq.Window('window'),
                         rq.Card32('selection'),
                         rq.Card32('mask')
                         )

def select_selection_input(self, window, selection, mask):
    return SelectSelectionInput(opcode=self.display.get_extension_major(extname),
                                display=self.display,
                                window=window,
                                selection=selection,
                                mask=mask)


class SelectionNotify(rq.Event):
    _code = None
    _fields = rq.Struct(rq.Card8('type'),
                        rq.Card8('sub_code'),
                        rq.Card16('sequence_number'),
                        rq.Window('window'),
                        rq.Window('owner'),
                        rq.Card32('selection'),
                        rq.Card32('timestamp'),
                        rq.Card32('selection_timestamp'),
                        rq.Pad(8))


class SetSelectionOwnerNotify(SelectionNotify):
    pass


class SelectionWindowDestroyNotify(SelectionNotify):
    pass


class SelectionClientCloseNotify(SelectionNotify):
    pass


class SelectCursorInput(rq.Request):
    _request = rq.Struct(rq.Card8('opcode'),
                         rq.Opcode(3),
                         rq.RequestLength(),
                         rq.Window('window'),
                         rq.Card32('mask')
                         )

def select_cursor_input(self, window, mask):
    return SelectCursorInput(opcode=self.display.get_extension_major(extname),
                             display=self.display,
                             window=window,
                             cursor_serial=0,
                             mask=mask)


class GetCursorImage(rq.ReplyRequest):
    _request = rq.Struct(rq.Card8('opcode'),
                         rq.Opcode(4),
                         rq.RequestLength()
                         )
    _reply = rq.Struct(rq.ReplyCode(),
                       rq.Pad(1),
                       rq.Card16('sequence_number'),
                       rq.ReplyLength(),
                       rq.Int16('x'),
                       rq.Int16('y'),
                       rq.Card16('width'),
                       rq.Card16('height'),
                       rq.Card16('xhot'),
                       rq.Card16('yhot'),
                       rq.Card32('cursor_serial'),
                       rq.Pad(8),
                       rq.List('cursor_image', rq.Card32)
                       )

def get_cursor_image(self, window):
    return GetCursorImage(opcode=self.display.get_extension_major(extname),
                          display=self.display,
                         )


class DisplayCursorNotify(rq.Event):
    _code = None
    _fields = rq.Struct(rq.Card8('type'),
                        rq.Card8('sub_code'),
                        rq.Card16('sequence_number'),
                        rq.Window('window'),
                        rq.Card32('cursor_serial'),
                        rq.Card32('timestamp'))


def init(disp, info):
    disp.extension_add_method('display', 'xfixes_select_selection_input', select_selection_input)
    disp.extension_add_method('display', 'xfixes_query_version', query_version)
    disp.extension_add_method('window', 'xfixes_hide_cursor', hide_cursor)
    disp.extension_add_method('window', 'xfixes_show_cursor', show_cursor)
    disp.extension_add_method('display', 'xfixes_select_cursor_input', select_cursor_input)
    disp.extension_add_method('display', 'xfixes_get_cursor_image', get_cursor_image)

    disp.extension_add_subevent(info.first_event + XFixesSelectionNotify, XFixesSetSelectionOwnerNotify, SetSelectionOwnerNotify)
    disp.extension_add_subevent(info.first_event + XFixesSelectionNotify, XFixesSelectionWindowDestroyNotify, SelectionWindowDestroyNotify)
    disp.extension_add_subevent(info.first_event + XFixesSelectionNotify, XFixesSelectionClientCloseNotify, SelectionClientCloseNotify)
    disp.extension_add_subevent(info.first_event + XFixesCursorNotify, XFixesDisplayCursorNotify, DisplayCursorNotify)



# Xlib.ext.ge -- Generic Event extension module
#
#    Copyright (C) 2012 Outpost Embedded, LLC
#      Forest Bond <forest.bond@rapidrollout.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

'''
ge - Generic Event Extension
'''

from Xlib.protocol import rq

extname = 'Generic Event Extension'


GenericEventCode = 35


class GEQueryVersion(rq.ReplyRequest):
    _request = rq.Struct(
        rq.Card8('opcode'),
        rq.Opcode(0),
        rq.RequestLength(),
        rq.Card32('major_version'),
        rq.Card32('minor_version'),
        )
    _reply = rq.Struct(
        rq.ReplyCode(),
        rq.Pad(1),
        rq.Card16('sequence_number'),
        rq.ReplyLength(),
        rq.Card32('major_version'),
        rq.Card32('minor_version'),
        rq.Pad(16),
        )


def query_version(self):
    return GEQueryVersion(
        display=self.display,
        opcode=self.display.get_extension_major(extname),
        major_version=1,
        minor_version=0,
        )


class GenericEvent(rq.Event):
    _code = GenericEventCode
    _fields = rq.Struct(
        rq.Card8('type'),
        rq.Card8('extension'),
        rq.Card16('sequence_number'),
        rq.Card32('length'),
        rq.Card16('evtype'),
        # Some generic events make use of this space, but with
        # others the data is simply discarded.  In any case we
        # don't need to explicitly pad this out as we are
        # always given at least 32 bytes and we save
        # everything after the first ten as the "data" field.
        #rq.Pad(22),
        )

    def __init__(self, binarydata = None, display = None, **keys):
        if binarydata:
            data = binarydata[10:]
            binarydata = binarydata[:10]
        else:
            data = ''

        rq.Event.__init__(
            self,
            binarydata=binarydata,
            display=display,
            **keys
            )

        if display:
            ge_event_data = getattr(display, 'ge_event_data', None)
            if ge_event_data:
                estruct = ge_event_data.get((self.extension, self.evtype), None)
                if estruct:
                    data, _ = estruct.parse_binary(data, display)

        self._data['data'] = data


def add_event_data(self, extension, evtype, estruct):
    if not hasattr(self.display, 'ge_event_data'):
        self.display.ge_event_data = {}
    self.display.ge_event_data[(extension, evtype)] = estruct


def init(disp, info):
    disp.extension_add_method('display', 'ge_query_version', query_version)
    disp.extension_add_method('display', 'ge_add_event_data', add_event_data)
    disp.extension_add_event(GenericEventCode, GenericEvent)



# Xlib.ext.__init__ -- X extension modules
#
#    Copyright (C) 2000 Peter Liljenberg <petli@ctrl-c.liu.se>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place,
#    Suite 330,
#    Boston, MA 02111-1307 USA

# __extensions__ is a list of tuples: (extname, extmod)
# extname is the name of the extension according to the X
# protocol.  extmod is the name of the module in this package.

__extensions__ = [
    # We load this first so other extensions can register generic event data
    # structures.
    ('Generic Event Extension', 'ge'),
    ('XTEST', 'xtest'),
    ('SHAPE', 'shape'),
    ('XINERAMA', 'xinerama'),
    ('RECORD', 'record'),
    ('Composite', 'composite'),
    ('RANDR', 'randr'),
    ('XFIXES', 'xfixes'),
    ('SECURITY', 'security'),
    ('XInputExtension', 'xinput'),
    ('NV-CONTROL', 'nvcontrol'),
    ('DAMAGE', 'damage'),
    ('DPMS', 'dpms'),
    ('X-Resource', 'res'),
    ('MIT-SCREEN-SAVER', 'screensaver'),
    ]

__all__ = map(lambda x: x[1], __extensions__)
